from django.core.management.base import NoArgsCommand
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    PrivateFormat,
    PublicFormat,
    NoEncryption
)


class Command(NoArgsCommand):
    help = "Generate a public / private RSA key pair"

    def handle_noargs(self, **options):
        private = rsa.generate_private_key(
            public_exponent=65537,
            key_size=options.get('bits', 2048),
            backend=default_backend()
        )
        public = private.public_key()

        print private.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption())
        print public.public_bytes(Encoding.PEM, PublicFormat.SubjectPublicKeyInfo)



from django.db import models
from django.contrib.auth.models import User


class PublicKey(models.Model):
    user = models.ForeignKey(User, related_name='public_keys')
    key = models.TextField(help_text="The user's RSA public key")



import unittest
import time
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    PrivateFormat,
    PublicFormat,
    NoEncryption
)

import asymmetric_jwt_auth.token as token


class AuthTest(unittest.TestCase):
    def generate_key_pair(self, size=2048):
        private = rsa.generate_private_key(
            public_exponent=65537,
            key_size=size,
            backend=default_backend()
        )
        public = private.public_key()

        pem_private = private.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption())
        pem_public = public.public_bytes(Encoding.PEM, PublicFormat.SubjectPublicKeyInfo)
        return pem_private, pem_public

    def test_roundtrip(self):
        private, public = self.generate_key_pair()
        t = token.sign('guido', private)
        token_data = token.verify(t, public)
        self.assertTrue(token_data)
        self.assertEqual(token_data.get('username'), 'guido')

    def test_bad_keys(self):
        private1, public1 = self.generate_key_pair()
        private2, public2 = self.generate_key_pair()

        t = token.sign('guido', private1)
        token_data = token.verify(t, public1)
        self.assertTrue(token_data)

        t = token.sign('guido', private2)
        token_data = token.verify(t, public2)
        self.assertTrue(token_data)

        t = token.sign('guido', private1)
        token_data = token.verify(t, public2)
        self.assertFalse(token_data)

        t = token.sign('guido', private2)
        token_data = token.verify(t, public1)
        self.assertFalse(token_data)

    def test_bad_iat(self):
        private, public = self.generate_key_pair()

        t = token.sign('guido', private, iat=time.time())
        token_data = token.verify(t, public)
        self.assertTrue(token_data)

        # IAT tolerance exists to account for clock drift between disparate systems.
        tolerance = token.TIMESTAMP_TOLERANCE + 1

        t = token.sign('guido', private, iat=time.time() - tolerance)
        token_data = token.verify(t, public)
        self.assertFalse(token_data)

        t = token.sign('guido', private, iat=time.time() + tolerance)
        token_data = token.verify(t, public)
        self.assertFalse(token_data)

    def test_bad_nonce(self):
        private, public = self.generate_key_pair()

        t = token.sign('guido', private, generate_nonce=lambda username, iat: 1)
        token_data = token.verify(t, public, validate_nonce=lambda username, iat, nonce: nonce == 1)
        self.assertTrue(token_data)

        t = token.sign('guido', private, generate_nonce=lambda username, iat: 1)
        token_data = token.verify(t, public, validate_nonce=lambda username, iat, nonce: nonce == 2)
        self.assertFalse(token_data)

        t = token.sign('guido', private, generate_nonce=lambda username, iat: 2)
        token_data = token.verify(t, public, validate_nonce=lambda username, iat, nonce: nonce == 1)
        self.assertFalse(token_data)



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    PrivateFormat,
    NoEncryption
)
import os.path
import asymmetric_jwt_auth.token as token


def load_private_key(key_file, key_password=None):
    key_file = os.path.expanduser(key_file)
    key_file = os.path.abspath(key_file)
    with open(key_file, 'rU') as key:
        private = serialization.load_pem_private_key(
            key.read(),
            password=key_password,
            backend=default_backend()
        )
        private = private.private_bytes(Encoding.PEM, PrivateFormat.PKCS8, NoEncryption())
    return private


def create_auth_header(username, key_file="~/.ssh/id_rsa", key_password=None):
    """Create an HTTP Authorization header using a private key file

    username - The username to authenticate as on the remote system
    key_file - Path to a file containing the user's private key. Defaults
               to ~/.ssh/id_rsa. Should be in PEM format.
    """
    key = load_private_key(key_file, key_password)
    claim = token.sign(username, key)
    return "JWT %s" % claim
